import React, {useState, useEffect} from 'react'
import {useSelector, useDispatch} from 'react-redux'

import ShippingAddress from './ShippingAddress.js'
import PaymentMethod from './PaymentMethod.js'
import ReviewOrder from './ReviewOrder.js'
import SubmitOrder from './SubmitOrder.js'
import './checkout.css'

import {dispatchOrderInfo} from './../../actions/checkout_actions'





export default function Checkout() {
    
    ///////////////////////
    // Shipping Info State
    ///////////////////////
    const [firstName, setFirstName] = useState([])
    const [lastName, setLastName] = useState([])
    const [shippingAddress, setShippingAddress] = useState([])
    const [aptSuiteNo, setAptSuiteNo] = useState([])
    const [city, setCity] = useState([])
    const [state, setState] = useState([])
    const [zipCode, setZipCode] = useState([])


    ///////////////////////
    // Billing Info State
    ///////////////////////
    const [cardNumber, setCardNumber] = useState([])
    const [cardHolderName, setCardHolderName] = useState([])
    const [expirationDate, setExpirationDate] = useState([])
    const [cvv, setCVV] = useState([])


    ///////////////////////
    // Order Review / Summary State
    ///////////////////////
    const [numCartItems, setNumCartItems] = useState([3])
    const [itemsTotal, setItemsTotal] = useState([87.65]) // arbitrary cart total
    const [shippingHandling, setShippingHandling] = useState([8.00]) // arbitraty shipping and handling amount
    const [taxRate, setTaxRate] = useState([0.0725]) // california tax rate


    ///////////////////////
    // Submit Button State
    ///////////////////////
    const [submitButtonActive, setSubmitButtonActive] = useState([0])
    const [allFieldsValidated, setAllFieldsValidated] = useState([0])
    const [errorWhileValidating, setErrorWhileValidating] = useState([0])


    ///////////////////////
    // Submit Action State
    ///////////////////////
    
    const dispatch = useDispatch()




    // This effect listens for the submit button to be clicked
    // It then checks each of the required fields to make sure they are non null
    useEffect( () => {

        // when submit button clicked after being inactive
        if (submitButtonActive === 1) {
            
            // fields to make sure are non-empty
            let fieldsToCheck = [
                firstName,
                lastName,
                shippingAddress,
                city,
                state,
                zipCode,
                cardNumber,
                cardHolderName,
                expirationDate,
                cvv
            ]
    
            // initially validated, gets flipped if required field is empty
            let validated = 1
            for (let i in fieldsToCheck) {
                if (fieldsToCheck[i].length === 0) {
                    console.log("a required Field Value was empty")
                    validated = 0
                    break
                }
            }
    
            // if validated, propogate to component state that all fields were validated
            if (validated === 1) {
                setAllFieldsValidated(1)
            }

            // if invalid, propogate to component state
            else {
                console.log("Failed to validate")
                setErrorWhileValidating(1)
                setSubmitButtonActive(0)
            }
        }

        // when submit button gets reset from active to inactive
        else if (submitButtonActive === 0) {
            ;
        }

    }, [submitButtonActive])



    // This effect looks for the allFieldsValidated signal and fires off the order submission when complete
    useEffect( () => {
        handleSubmit()
        console.log("Successful submission request")
    }, [allFieldsValidated])



    let handleSubmit = () => {
        console.log("Submitting Order!")
        // let order = {}
        // order["firstName"] = firstName
        // order["lastName"] = lastName
        // order["shippingAddress"] = shippingAddress
        // order["aptSuitNo"] = aptSuiteNo
        // order["city"] = city
        // order["state"] = state
        // order["zipCode"] = zipCode
        // order["cardNumber"] = cardNumber
        // order["cardHolderName"] = cardHolderName
        // order["expirationDate"] = expirationDate
        // order["cvv"] = cvv
        // console.log(order)

        let chrisSpec = {
            "user": {
                "userId": "e-com-test-id",
                "email": "test@test.test",
                "firstName":"ecom",
                "lastName":"test"
            },
            "address": {
                "state": "IL",
                "city": "chicago",
                "zip": "90210",
                "street": "Grand"
            },
            "cart": {
                "id": "something",
                "userId": "e-com-test-id",
                "cartItems":[
                        {
                            "quantity": 2,
                            "productRef": {
                                "id": "something",
                                "sku": "200000",
                                "upc": "00002"
                            }
                        }
                    ]
            }   
        }

        dispatch(dispatchOrderInfo(chrisSpec))

    }




    // This watches for changes on any shipping input fields and prints the current states on any change
    useEffect( () => {
        console.log("======================================")
        console.log("First Name: " + firstName)
        console.log("Last Name: " + lastName)
        console.log("Shipping Address: " + shippingAddress)
        console.log("Apt Suite No: " + aptSuiteNo)
        console.log("City: " + city)
        console.log("State: " + state)
        console.log("Zip: " + zipCode)
    }, [firstName, lastName, shippingAddress, aptSuiteNo, city, state, zipCode])


    // This watches for changes on any billing input fields and prints the current states on any change
    useEffect( () => {
        console.log("======================================")
        console.log("Card Number: " + cardNumber)
        console.log("Card Holder Name: " + cardHolderName)
        console.log("Expiration Date: " + expirationDate)
        console.log("CVV: " + cvv)
    }, [cardNumber, cardHolderName, expirationDate, cvv])


    return (
        <div id="checkout-container">

            {/* Collects User's shipping info */}
            <ShippingAddress 
                captureFirstName={setFirstName}
                captureLastName={setLastName}
                captureShippingAddress={setShippingAddress}
                captureAptSuiteNo={setAptSuiteNo}
                captureCity={setCity}
                captureState={setState}
                captureZipCode={setZipCode}
            />

            {/* Collects User's payment info */}
            <PaymentMethod 
                captureCardNumber={setCardNumber}
                captureCardHolderName={setCardHolderName}
                captureExpirationDate={setExpirationDate}
                captureCVV={setCVV}
            />

            {/* Displays info about order. Calculates price with tax / shipping applied */}
            <ReviewOrder
                numCartItems={numCartItems}
                itemsTotal={itemsTotal}
                shippingHandling={shippingHandling}
                taxRate={taxRate}
            />

            {/* Request to submit happens here, initiates input validation and sends if success */}
            <SubmitOrder
                submitButtonActive={submitButtonActive}
                setSubmitButtonActive={setSubmitButtonActive}
                errorWhileValidating={errorWhileValidating}
                allFieldsValidated={allFieldsValidated}
            />
            
        </div>
    )
}


